@@ -19,112 +19,84 @@ Overview of Estimator
 
 Estimator class estimates expectation values of quantum circuits and observables.
 
-An estimator object is initialized with multiple quantum circuits and observables
-and users can specify pairs of quantum circuits and observables
-to estimate the expectation values.
-
-An estimator is initialized with the following elements.
+An estimator is initialized with an empty parameter set. The estimator is used to
+create a :class:`~qiskit.providers.JobV1`, via the
+:meth:`qiskit.primitives.Estimator.run()` method. This method is called
+with the following parameters
 
 * quantum circuits (:math:`\psi_i(\theta)`): list of (parameterized) quantum circuits
-  (a list of :class:`~qiskit.circuit.QuantumCircuit`))
+  (a list of :class:`~qiskit.circuit.QuantumCircuit` objects).
 
-* observables (:math:`H_j`): a list of :class:`~qiskit.quantum_info.SparsePauliOp`.
-
-The estimator is called with the following inputs.
-
-* circuit indexes: a list of indexes of the quantum circuits.
-
-* observable indexes: a list of indexes of the observables.
-
-* parameters: a list of parameters of the quantum circuits.
-  (:class:`~qiskit.circuit.parametertable.ParameterView` or
-  a list of :class:`~qiskit.circuit.Parameter`).
+* observables (:math:`H_j`): a list of :class:`~qiskit.quantum_info.SparsePauliOp`
+  objects.
 
 * parameter values (:math:`\theta_k`): list of sets of values
-  to be bound to the parameters of the quantum circuits.
-  (list of list of float)
+  to be bound to the parameters of the quantum circuits
+  (list of list of float).
 
-The output is an :class:`~qiskit.primitives.EstimatorResult` which contains a list of
-expectation values plus optional metadata like confidence intervals for the estimation.
+The method returns a :class:`~qiskit.providers.JobV1` object, calling
+:meth:`qiskit.providers.JobV1.result()` yields the
+a list of expectation values plus optional metadata like confidence intervals for
+the estimation.
 
 .. math::
 
     \langle\psi_i(\theta_k)|H_j|\psi_i(\theta_k)\rangle
 
-
-The estimator object is expected to be ``close()`` d after use or
-accessed inside "with" context
-and the objects are called with parameter values and run options
-(e.g., ``shots`` or number of shots).
-
-Here is an example of how estimator is used.
+Here is an example of how the estimator is used.
 
 .. code-block:: python
 
+    from qiskit.primitives import Estimator
     from qiskit.circuit.library import RealAmplitudes
     from qiskit.quantum_info import SparsePauliOp
 
     psi1 = RealAmplitudes(num_qubits=2, reps=2)
     psi2 = RealAmplitudes(num_qubits=2, reps=3)
 
-    params1 = psi1.parameters
-    params2 = psi2.parameters
-
     H1 = SparsePauliOp.from_list([("II", 1), ("IZ", 2), ("XI", 3)])
     H2 = SparsePauliOp.from_list([("IZ", 1)])
     H3 = SparsePauliOp.from_list([("ZI", 1), ("ZZ", 1)])
 
-    with Estimator([psi1, psi2], [H1, H2, H3], [params1, params2]) as e:
-        theta1 = [0, 1, 1, 2, 3, 5]
-        theta2 = [0, 1, 1, 2, 3, 5, 8, 13]
-        theta3 = [1, 2, 3, 4, 5, 6]
+    theta1 = [0, 1, 1, 2, 3, 5]
+    theta2 = [0, 1, 1, 2, 3, 5, 8, 13]
+    theta3 = [1, 2, 3, 4, 5, 6]
 
-        # calculate [ <psi1(theta1)|H1|psi1(theta1)> ]
-        result = e([0], [0], [theta1])
-        print(result)
+    estimator = Estimator()
 
-        # calculate [ <psi1(theta1)|H2|psi1(theta1)>, <psi1(theta1)|H3|psi1(theta1)> ]
-        result2 = e([0, 0], [1, 2], [theta1]*2)
-        print(result2)
+    # calculate [ <psi1(theta1)|H1|psi1(theta1)> ]
+    job = estimator.run([psi1], [H1], [theta1])
+    job_result = job.result() # It will block until the job finishes.
+    print(f"The primitive-job finished with result {job_result}"))
 
-        # calculate [ <psi2(theta2)|H2|psi2(theta2)> ]
-        result3 = e([1], [1], [theta2])
-        print(result3)
-
-        # calculate [ <psi1(theta1)|H1|psi1(theta1)>, <psi1(theta3)|H1|psi1(theta3)> ]
-        result4 = e([0, 0], [0, 0], [theta1, theta3])
-        print(result4)
-
-        # calculate [ <psi1(theta1)|H1|psi1(theta1)>,
-        #             <psi2(theta2)|H2|psi2(theta2)>,
-        #             <psi1(theta3)|H3|psi1(theta3)> ]
-        result5 = e([0, 1, 0], [0, 1, 2], [theta1, theta2, theta3])
-        print(result5)
-
-        # Objects can be passed instead of indices.
-        # calculate [ <psi2(theta2)|H2|psi2(theta2)> ]
-        # Note that passing objects has an overhead
-        # since the corresponding indices need to be searched.
-        result6 = e([psi2], [H2], [theta2])
-        print(result6)
+    # calculate [ <psi1(theta1)|H1|psi1(theta1)>,
+    #             <psi2(theta2)|H2|psi2(theta2)>,
+    #             <psi1(theta3)|H3|psi1(theta3)> ]
+    job2 = estimator.run([psi1, psi2, psi1], [H1, H2, H3], [theta1, theta2, theta3])
+    job_result = job2.result()
+    print(f"The primitive-job finished with result {job_result}")
 """
 from __future__ import annotations
 
 from abc import ABC, abstractmethod
 from collections.abc import Iterable, Sequence
 from copy import copy
-from typing import Any
+from typing import cast, Any
+from warnings import warn
 
 import numpy as np
 
 from qiskit.circuit import Parameter, QuantumCircuit
 from qiskit.circuit.parametertable import ParameterView
-from qiskit.exceptions import QiskitError
+from qiskit.opflow import PauliSumOp
+from qiskit.providers import JobV1 as Job
+from qiskit.providers import Options
 from qiskit.quantum_info.operators import SparsePauliOp
-from qiskit.utils.deprecation import deprecate_arguments
+from qiskit.quantum_info.operators.base_operator import BaseOperator
+from qiskit.utils.deprecation import deprecate_arguments, deprecate_function
 
 from .estimator_result import EstimatorResult
-from .utils import _finditer
+from .utils import _circuit_key, _observable_key, init_observable
 
 
 class BaseEstimator(ABC):
@@ -133,12 +105,15 @@ class BaseEstimator(ABC):
     Base class for Estimator that estimates expectation values of quantum circuits and observables.
     """
 
+    __hash__ = None
+
     def __init__(
         self,
-        circuits: Iterable[QuantumCircuit] | QuantumCircuit,
-        observables: Iterable[SparsePauliOp] | SparsePauliOp,
+        circuits: Iterable[QuantumCircuit] | QuantumCircuit | None = None,
+        observables: Iterable[SparsePauliOp] | SparsePauliOp | None = None,
         parameters: Iterable[Iterable[Parameter]] | None = None,
-    ):
+        options: dict | None = None,
+    ) -> None:
         """
         Creating an instance of an Estimator, or using one in a ``with`` context opens a session that
         holds resources until the instance is ``close()`` ed or the context is exited.
@@ -150,68 +125,98 @@ class BaseEstimator(ABC):
                 will be bound. Defaults to ``[circ.parameters for circ in circuits]``
                 The indexing is such that ``parameters[i, j]`` is the j-th formal parameter of
                 ``circuits[i]``.
+            options: Default options.
 
         Raises:
-            QiskitError: For mismatch of circuits and parameters list.
+            ValueError: For mismatch of circuits and parameters list.
         """
+        if circuits is not None or observables is not None or parameters is not None:
+            warn(
+                "The BaseEstimator 'circuits', `observables`, `parameters` kwarg are deprecated "
+                "as of 0.22.0 and will be removed no earlier than 3 months after the "
+                "release date. You can use 'run' method to append objects.",
+                DeprecationWarning,
+                2,
+            )
         if isinstance(circuits, QuantumCircuit):
             circuits = (circuits,)
-        self._circuits = tuple(circuits)
+        self._circuits = [] if circuits is None else list(circuits)
 
         if isinstance(observables, SparsePauliOp):
             observables = (observables,)
-        self._observables = tuple(observables)
+        self._observables = [] if observables is None else list(observables)
 
         # To guarantee that they exist as instance variable.
         # With only dynamic set, the python will not know if the attribute exists or not.
-        self._circuit_ids = self._circuit_ids  # type: ignore
-        self._observable_ids = self._observable_ids  # type: ignore
+        self._circuit_ids: dict[tuple, int] = self._circuit_ids
+        self._observable_ids: dict[tuple, int] = self._observable_ids
 
         if parameters is None:
-            self._parameters = tuple(circ.parameters for circ in self._circuits)
+            self._parameters = [circ.parameters for circ in self._circuits]
         else:
-            self._parameters = tuple(ParameterView(par) for par in parameters)
-            if len(self._parameters) != len(self._circuits):  # type: ignore
-                raise QiskitError(
+            self._parameters = [ParameterView(par) for par in parameters]
+            if len(self._parameters) != len(self._circuits):
+                raise ValueError(
                     f"Different number of parameters ({len(self._parameters)}) and "
-                    f"circuits ({len(self._circuits)})"  # type: ignore
+                    f"circuits ({len(self._circuits)})"
                 )
             for i, (circ, params) in enumerate(zip(self._circuits, self._parameters)):
                 if circ.num_parameters != len(params):
-                    raise QiskitError(
+                    raise ValueError(
                         f"Different numbers of parameters of {i}-th circuit: "
                         f"expected {circ.num_parameters}, actual {len(params)}."
                     )
+        self._run_options = Options()
+        if options is not None:
+            self._run_options.update_options(**options)
 
-    def __new__(  # pylint: disable=unused-argument
-        cls,
-        circuits: Iterable[QuantumCircuit] | QuantumCircuit,
-        observables: Iterable[SparsePauliOp] | SparsePauliOp,
-        *args: Any,
-        parameters: Iterable[Iterable[Parameter]] | None = None,
-        **kwargs: Any,
+    def __new__(
+        cls,  # pylint: disable=unused-argument
+        circuits: Iterable[QuantumCircuit] | QuantumCircuit | None = None,
+        observables: Iterable[SparsePauliOp] | SparsePauliOp | None = None,
+        parameters: Iterable[Iterable[Parameter]]  # pylint: disable=unused-argument
+        | None = None,  # pylint: disable=unused-argument
+        **kwargs: Any,  # pylint: disable=unused-argument
     ) -> BaseEstimator:
 
         self = super().__new__(cls)
-        if isinstance(circuits, Iterable):
+        if circuits is None:
+            self._circuit_ids = {}
+        elif isinstance(circuits, Iterable):
             circuits = copy(circuits)
-            self._circuit_ids = [id(circuit) for circuit in circuits]
+            self._circuit_ids = {
+                _circuit_key(circuit): i for i, circuit in enumerate(circuits)
+            }
         else:
-            self._circuit_ids = [id(circuits)]
-        if isinstance(observables, Iterable):
+            self._circuit_ids = {_circuit_key(circuits): 0}
+        if observables is None:
+            self._observable_ids = {}
+        elif isinstance(observables, Iterable):
             observables = copy(observables)
-            self._observable_ids = [id(observable) for observable in observables]
+            self._observable_ids = {
+                _observable_key(init_observable(observable)): i
+                for i, observable in enumerate(observables)
+            }
         else:
-            self._observable_ids = [id(observables)]
+            self._observable_ids = {_observable_key(init_observable(observables)): 0}
         return self
 
+    @deprecate_function(
+        "The BaseEstimator.__enter__ method is deprecated as of Qiskit Terra 0.22.0 "
+        "and will be removed no sooner than 3 months after the releasedate. "
+        "BaseEstimator should be initialized directly.",
+    )
     def __enter__(self) -> BaseEstimator:
         return self
 
+    @deprecate_function(
+        "The BaseEstimator.__exit__ method is deprecated as of Qiskit Terra 0.22.0 "
+        "and will be removed no sooner than 3 months after the releasedate. "
+        "BaseEstimator should be initialized directly.",
+    )
     def __exit__(self, *exc_info: Any) -> None:
         self.close()
 
-    @abstractmethod
     def close(self) -> None:
         """Close the session and free resources"""
         ...
@@ -223,7 +228,7 @@ class BaseEstimator(ABC):
         Returns:
             The quantum circuits.
         """
-        return self._circuits  # type: ignore
+        return tuple(self._circuits)
 
     @property
     def observables(self) -> tuple[SparsePauliOp, ...]:
@@ -232,7 +237,7 @@ class BaseEstimator(ABC):
         Returns:
             The observables.
         """
-        return self._observables  # type: ignore
+        return tuple(self._observables)
 
     @property
     def parameters(self) -> tuple[ParameterView, ...]:
@@ -241,8 +246,30 @@ class BaseEstimator(ABC):
         Returns:
             Parameters, where ``parameters[i][j]`` is the j-th parameter of the i-th circuit.
         """
-        return self._parameters
+        return tuple(self._parameters)
 
+    @property
+    def options(self) -> Options:
+        """Return options values for the estimator.
+
+        Returns:
+            options
+        """
+        return self._run_options
+
+    def set_options(self, **fields: Any) -> None:
+        """Set options values for the estimator.
+
+        Args:
+            **fields: The fields to update the options
+        """
+        self._run_options.update_options(**fields)
+
+    @deprecate_function(
+        "The BaseSampler.__call__ method is deprecated as of Qiskit Terra 0.22.0 "
+        "and will be removed no sooner than 3 months after the releasedate. "
+        "Use run method instead.",
+    )
     @deprecate_arguments(
         {"circuit_indices": "circuits", "observable_indices": "observables"}
     )
@@ -278,14 +305,14 @@ class BaseEstimator(ABC):
             circuits: the list of circuit indices or circuit objects.
             observables: the list of observable indices or observable objects.
             parameter_values: concrete parameters to be bound.
-            run_options: runtime options used for circuit execution.
+            run_options: Default runtime options used for circuit execution.
 
         Returns:
             EstimatorResult: The result of the estimator.
 
         Raises:
-            QiskitError: For mismatch of object id.
-            QiskitError: For mismatch of length of Sequence.
+            ValueError: For mismatch of object id.
+            ValueError: For mismatch of length of Sequence.
         """
 
         # Support ndarray
@@ -293,36 +320,37 @@ class BaseEstimator(ABC):
             parameter_values = parameter_values.tolist()
 
         # Allow objects
-        try:
-            circuits = [
-                next(_finditer(id(circuit), self._circuit_ids))  # type: ignore
-                if not isinstance(circuit, (int, np.integer))
-                else circuit
-                for circuit in circuits
-            ]
-        except StopIteration as err:
-            raise QiskitError(
+        circuits = [
+            self._circuit_ids.get(_circuit_key(circuit))
+            if not isinstance(circuit, (int, np.integer))
+            else circuit
+            for circuit in circuits
+        ]
+        if any(circuit is None for circuit in circuits):
+            raise ValueError(
                 "The circuits passed when calling estimator is not one of the circuits used to "
                 "initialize the session."
-            ) from err
-        try:
-            observables = [
-                next(_finditer(id(observable), self._observable_ids))
-                if not isinstance(observable, (int, np.integer))
-                else observable
-                for observable in observables
-            ]
-        except StopIteration as err:
-            raise QiskitError(
+            )
+        observables = [
+            self._observable_ids.get(_observable_key(observable))
+            if not isinstance(observable, (int, np.integer))
+            else observable
+            for observable in observables
+        ]
+        if any(observable is None for observable in observables):
+            raise ValueError(
                 "The observables passed when calling estimator is not one of the observables used to "
                 "initialize the session."
-            ) from err
+            )
+
+        circuits = cast("list[int]", circuits)
+        observables = cast("list[int]", observables)
 
         # Allow optional
         if parameter_values is None:
             for i in circuits:
-                if len(self._circuits[i].parameters) != 0:  # type: ignore
-                    raise QiskitError(
+                if len(self._circuits[i].parameters) != 0:
+                    raise ValueError(
                         f"The {i}-th circuit is parameterised,"
                         "but parameter values are not given."
                     )
@@ -330,19 +358,19 @@ class BaseEstimator(ABC):
 
         # Validation
         if len(circuits) != len(observables):
-            raise QiskitError(
+            raise ValueError(
                 f"The number of circuits ({len(circuits)}) does not match "
                 f"the number of observables ({len(observables)})."
             )
         if len(circuits) != len(parameter_values):
-            raise QiskitError(
+            raise ValueError(
                 f"The number of circuits ({len(circuits)}) does not match "
                 f"the number of parameter value sets ({len(parameter_values)})."
             )
 
         for i, value in zip(circuits, parameter_values):
             if len(value) != len(self._parameters[i]):
-                raise QiskitError(
+                raise ValueError(
                     f"The number of values ({len(value)}) does not match "
                     f"the number of parameters ({len(self._parameters[i])}) for the {i}-th circuit."
                 )
@@ -351,28 +379,135 @@ class BaseEstimator(ABC):
             circuit_num_qubits = self.circuits[circ_i].num_qubits
             observable_num_qubits = self.observables[obs_i].num_qubits
             if circuit_num_qubits != observable_num_qubits:
-                raise QiskitError(
+                raise ValueError(
                     f"The number of qubits of the {circ_i}-th circuit ({circuit_num_qubits}) does "
                     f"not match the number of qubits of the {obs_i}-th observable "
                     f"({observable_num_qubits})."
                 )
 
         if max(circuits) >= len(self.circuits):
-            raise QiskitError(
+            raise ValueError(
                 f"The number of circuits is {len(self.circuits)}, "
                 f"but the index {max(circuits)} is given."
             )
         if max(observables) >= len(self.observables):
-            raise QiskitError(
+            raise ValueError(
                 f"The number of circuits is {len(self.observables)}, "
                 f"but the index {max(observables)} is given."
             )
+        run_opts = copy(self.options)
+        run_opts.update_options(**run_options)
 
         return self._call(
             circuits=circuits,
             observables=observables,
             parameter_values=parameter_values,
-            **run_options,
+            **run_opts.__dict__,
+        )
+
+    def run(
+        self,
+        circuits: QuantumCircuit | Sequence[QuantumCircuit],
+        observables: BaseOperator | PauliSumOp | Sequence[BaseOperator | PauliSumOp],
+        parameter_values: Sequence[float] | Sequence[Sequence[float]] | None = None,
+        **run_options: Any,
+    ) -> Job:
+        """Run the job of the estimation of expectation value(s).
+
+        ``circuits``, ``observables``, and ``parameter_values`` should have the same
+        length. The i-th element of the result is the expectation of observable
+
+        .. code-block:: python
+
+            obs = observables[i]
+
+        for the state prepared by
+
+        .. code-block:: python
+
+            circ = circuits[i]
+
+        with bound parameters
+
+        .. code-block:: python
+
+            values = parameter_values[i].
+
+        Args:
+            circuits: one or more circuit objects.
+            observables: one or more observable objects.
+            parameter_values: concrete parameters to be bound.
+            **run_options: runtime options used for circuit execution.
+
+        Returns:
+            The job object of EstimatorResult.
+
+        Raises:
+            ValueError: Invalid arguments are given.
+        """
+        # Support ndarray
+        if isinstance(parameter_values, np.ndarray):
+            parameter_values = parameter_values.tolist()
+
+        if not isinstance(circuits, Sequence):
+            circuits = [circuits]
+        if not isinstance(observables, Sequence):
+            observables = [observables]
+        if (
+            len(circuits) > 0
+            and len(observables) > 0
+            and parameter_values is not None
+            and (
+                len(parameter_values) == 0
+                or not isinstance(parameter_values[0], (Sequence, Iterable))
+            )
+        ):
+            parameter_values = [parameter_values]  # type: ignore[assignment]
+
+        # Allow optional
+        if parameter_values is None:
+            for i, circuit in enumerate(circuits):
+                if circuit.num_parameters != 0:
+                    raise ValueError(
+                        f"The {i}-th circuit is parameterised,"
+                        "but parameter values are not given."
+                    )
+            parameter_values = [[]] * len(circuits)
+
+        # Validation
+        if len(circuits) != len(observables):
+            raise ValueError(
+                f"The number of circuits ({len(circuits)}) does not match "
+                f"the number of observables ({len(observables)})."
+            )
+        if len(circuits) != len(parameter_values):
+            raise ValueError(
+                f"The number of circuits ({len(circuits)}) does not match "
+                f"the number of parameter value sets ({len(parameter_values)})."
+            )
+
+        for i, (circuit, parameter_value) in enumerate(zip(circuits, parameter_values)):
+            if len(parameter_value) != circuit.num_parameters:  # type: ignore
+                raise ValueError(
+                    f"The number of values ({len(parameter_value)}) does not match "  # type: ignore
+                    f"the number of parameters ({circuit.num_parameters}) for the {i}-th circuit."
+                )
+
+        for i, (circuit, observable) in enumerate(zip(circuits, observables)):
+            if circuit.num_qubits != observable.num_qubits:
+                raise ValueError(
+                    f"The number of qubits of the {i}-th circuit ({circuit.num_qubits}) does "
+                    f"not match the number of qubits of the {i}-th observable "
+                    f"({observable.num_qubits})."
+                )
+        run_opts = copy(self.options)
+        run_opts.update_options(**run_options)
+
+        return self._run(
+            circuits,
+            observables,
+            parameter_values,  # type: ignore
+            **run_opts.__dict__,
         )
 
     @abstractmethod
@@ -384,3 +519,16 @@ class BaseEstimator(ABC):
         **run_options: Any,
     ) -> EstimatorResult:
         ...
+
+    # This will be comment out after 0.22. (This is necessary for the compatibility.)
+    # @abstractmethod
+    def _run(
+        self,
+        circuits: Sequence[QuantumCircuit],
+        observables: Sequence[BaseOperator | PauliSumOp],
+        parameter_values: Sequence[Sequence[float]],
+        **run_options: Any,
+    ) -> Job:
+        raise NotImplementedError(
+            "_run method is not implemented. This method will be @abstractmethod after 0.22."
+        )
