@@ -21,6 +21,7 @@ from collections import OrderedDict
 from typing import Dict, Callable, Optional, Union, List, Any, Type
 
 from qiskit.providers.backend import BackendV1 as Backend
+from qiskit.providers.provider import ProviderV1 as Provider
 from qiskit.providers.exceptions import QiskitBackendNotFoundError
 from qiskit.providers.providerutils import filter_backends
 
@@ -46,16 +47,30 @@ from .runtime_session import RuntimeSession  # pylint: disable=cyclic-import
 from .utils import RuntimeDecoder, to_base64_string, to_python_identifier
 from .utils.backend_decoder import configuration_from_server_data
 from .utils.hgp import to_instance_format, from_instance_format
-from .utils.utils import validate_job_tags
 from .api.client_parameters import ClientParameters
 from .runtime_options import RuntimeOptions
+from .utils.deprecation import (
+    deprecate_function,
+    issue_deprecation_msg,
+    deprecate_arguments,
+)
 
 logger = logging.getLogger(__name__)
 
 SERVICE_NAME = "runtime"
 
+DEPRECATED_PROGRAMS = [
+    "torch-train",
+    "torch-infer",
+    "sample-expval",
+    "quantum_kernal_alignment",
+    "sample-program",
+]
 
-class QiskitRuntimeService:
+_JOB_DEPRECATION_ISSUED = False
+
+
+class QiskitRuntimeService(Provider):
     """Class for interacting with the Qiskit Runtime service.
 
     Qiskit Runtime is a new architecture offered by IBM Quantum that
@@ -63,42 +78,41 @@ class QiskitRuntimeService:
     execute significantly faster within its improved hybrid quantum/classical
     process.
 
-    The Qiskit Runtime Service allows authorized users to upload their Qiskit
-    quantum programs. A Qiskit quantum program, also called a runtime program,
-    is a piece of Python code and its metadata that takes certain inputs, performs
-    quantum and maybe classical processing, and returns the results. The same or other
-    authorized users can invoke these quantum programs by simply passing in parameters.
-
     A sample workflow of using the runtime service::
 
-        from qiskit import QuantumCircuit
-        from qiskit_ibm_runtime import QiskitRuntimeService
+        from qiskit_ibm_runtime import QiskitRuntimeService, Session, Sampler, Estimator, Options
+        from qiskit.test.reference_circuits import ReferenceCircuits
+        from qiskit.circuit.library import RealAmplitudes
+        from qiskit.quantum_info import SparsePauliOp
 
+        # Initialize account.
         service = QiskitRuntimeService()
 
-        # Create a circuit.
-        qc = QuantumCircuit(2)
-        qc.h(0)
-        qc.cx(0, 1)
-        qc.measure_all()
+        # Set options, which can be overwritten at job level.
+        options = Options(optimization_level=1)
 
-        # Set the "sampler" program inputs
-        inputs = {
-            'circuits': qc,
-            'circuit_indices': [0]
-        }
+        # Prepare inputs.
+        bell = ReferenceCircuits.bell()
+        psi = RealAmplitudes(num_qubits=2, reps=2)
+        H1 = SparsePauliOp.from_list([("II", 1), ("IZ", 2), ("XI", 3)])
+        theta = [0, 1, 1, 2, 3, 5]
 
-        # Configure backend options
-        options = {'backend_name': "ibmq_qasm_simulator"}
+        with Session(service=service, backend="ibmq_qasm_simulator") as session:
+            # Submit a request to the Sampler primitive within the session.
+            sampler = Sampler(session=session, options=options)
+            job = sampler.run(circuits=bell)
+            print(f"Sampler results: {job.result()}")
 
-        # Execute the circuit using the "sampler" program.
-        job = service.run(program_id="sampler",
-                          options=options,
-                          inputs=inputs)
-        print(f"Job ID: {job.job_id}")
-        # Get runtime job result.
-        result = job.result()
-        print(result)
+            # Submit a request to the Estimator primitive within the session.
+            estimator = Estimator(session=session, options=options)
+            job = estimator.run(
+                circuits=[psi], observables=[H1], parameter_values=[theta]
+            )
+            print(f"Estimator results: {job.result()}")
+
+    The example above uses the dedicated :class:`~qiskit_ibm_runtime.Sampler`
+    and :class:`~qiskit_ibm_runtime.Estimator` classes. You can also
+    use the :meth:`run` method directly to invoke a Qiskit Runtime program.
 
     If the program has any interim results, you can use the ``callback``
     parameter of the :meth:`run` method to stream the interim results.
@@ -475,7 +489,7 @@ class QiskitRuntimeService:
                 return hgp
 
         raise QiskitBackendNotFoundError(
-            f"Backend {backend_name} cannot be found in any"
+            f"Backend {backend_name} cannot be found in any "
             f"hub/group/project for this account."
         )
 
@@ -488,6 +502,7 @@ class QiskitRuntimeService:
                 backend_name += "_"
             setattr(self, backend_name, backend)
 
+    # pylint: disable=arguments-differ
     def backends(
         self,
         name: Optional[str] = None,
@@ -685,6 +700,9 @@ class QiskitRuntimeService:
             raise QiskitBackendNotFoundError("No backend matches the criteria")
         return backends[0]
 
+    def get_backend(self, name: str = None, **kwargs: Any) -> Backend:
+        return self.backend(name, **kwargs)
+
     def pprint_programs(
         self,
         refresh: bool = False,
@@ -744,11 +762,17 @@ class QiskitRuntimeService:
                 # count is the total number of programs that would be returned if
                 # there was no limit or skip
                 count = response.get("count", 0)
+                if limit is None:
+                    limit = count
                 for prog_dict in program_page:
                     program = self._to_program(prog_dict)
                     self._programs[program.program_id] = program
-                if len(self._programs) == count:
-                    # Stop if there are no more programs returned by the server.
+                num_cached_programs = len(self._programs)
+                if num_cached_programs == count or num_cached_programs >= (
+                    limit + skip
+                ):
+                    # Stop if there are no more programs returned by the server or
+                    # if the number of cached programs is greater than the sum of limit and skip
                     break
                 offset += len(program_page)
         if limit is None:
@@ -840,8 +864,9 @@ class QiskitRuntimeService:
             program_id: Program ID.
             inputs: Program input parameters. These input values are passed
                 to the runtime program.
-            options: Runtime options that control the execution environment. See
-                :class:`RuntimeOptions` for all available options.
+            options: Runtime options that control the execution environment.
+                See :class:`RuntimeOptions` for all available options.
+
             callback: Callback function to be invoked for any interim results and final result.
                 The callback function will receive 2 positional parameters:
 
@@ -850,13 +875,13 @@ class QiskitRuntimeService:
 
             result_decoder: A :class:`ResultDecoder` subclass used to decode job results.
                 ``ResultDecoder`` is used if not specified.
-            instance: This is only supported for ``ibm_quantum`` runtime and is in the
+            instance: (DEPRECATED) This is only supported for ``ibm_quantum`` runtime and is in the
                 hub/group/project format.
             session_id: Job ID of the first job in a runtime session.
-            job_tags: Tags to be assigned to the job. The tags can subsequently be used
+            job_tags: (DEPRECATED) Tags to be assigned to the job. The tags can subsequently be used
                 as a filter in the :meth:`jobs()` function call.
-            max_execution_time: Maximum execution time in seconds. This overrides
-                the max_execution_time of the program and cannot exceed it.
+            max_execution_time: (DEPRECATED) Maximum execution time in seconds. This overrides
+                the max_execution_time of the program.
             start_session: Set to True to explicitly start a runtime session. Defaults to False.
 
         Returns:
@@ -867,45 +892,63 @@ class QiskitRuntimeService:
             RuntimeProgramNotFound: If the program cannot be found.
             IBMRuntimeError: An error occurred running the program.
         """
-        validate_job_tags(job_tags, IBMInputValueError)
-
-        if instance and self._channel != "ibm_quantum":
-            raise IBMInputValueError(
-                "The 'instance' keyword is only supported for ``ibm_quantum`` runtime. "
+        if program_id in DEPRECATED_PROGRAMS:
+            warnings.warn(
+                f"The {program_id} program has been deprecated as of qiskit-ibm-runtime 0.7 \
+                and will be removed no sooner than 1 month after the release date.",
+                DeprecationWarning,
+                stacklevel=2,
             )
 
+        qrt_options: RuntimeOptions = options
+        if options is None:
+            qrt_options = RuntimeOptions()
+        elif isinstance(options, Dict):
+            qrt_options = RuntimeOptions(**options)
+
+        deprecated = {
+            "instance": instance,
+            "job_tags": job_tags,
+            "max_execution_time": max_execution_time,
+        }
+        for name, param in deprecated.items():
+            if param is not None:
+                deprecate_arguments(
+                    deprecated=name,
+                    version="0.7",
+                    remedy=f'Please specify "{name}" inside "options".',
+                )
+                setattr(qrt_options, name, param)
+
         # If using params object, extract as dictionary
         if isinstance(inputs, ParameterNamespace):
             inputs.validate()
             inputs = vars(inputs)
 
-        if options is None:
-            options = RuntimeOptions()
-        if isinstance(options, dict):
-            options = RuntimeOptions(**options)
-
-        options.validate(channel=self.channel)
+        qrt_options.validate(channel=self.channel)
 
         backend = None
         hgp_name = None
         if self._channel == "ibm_quantum":
             # Find the right hgp
-            hgp = self._get_hgp(instance=instance, backend_name=options.backend_name)
-            backend = hgp.backend(options.backend_name)
+            hgp = self._get_hgp(
+                instance=qrt_options.instance, backend_name=qrt_options.backend
+            )
+            backend = hgp.backend(qrt_options.backend)
             hgp_name = hgp.name
 
         result_decoder = result_decoder or ResultDecoder
         try:
             response = self._api_client.program_run(
                 program_id=program_id,
-                backend_name=options.backend_name,
+                backend_name=qrt_options.backend,
                 params=inputs,
-                image=options.image,
+                image=qrt_options.image,
                 hgp=hgp_name,
-                log_level=options.log_level,
+                log_level=qrt_options.log_level,
                 session_id=session_id,
-                job_tags=job_tags,
-                max_execution_time=max_execution_time,
+                job_tags=qrt_options.job_tags,
+                max_execution_time=qrt_options.max_execution_time,
                 start_session=start_session,
             )
         except RequestsApiError as ex:
@@ -918,6 +961,18 @@ class QiskitRuntimeService:
         if not backend:
             backend = self.backend(name=response["backend"])
 
+        global _JOB_DEPRECATION_ISSUED  # pylint: disable=global-statement
+        if not (start_session or session_id) and not _JOB_DEPRECATION_ISSUED:
+            # No need to issue warning if session is used since the old style session
+            # didn't return a job, and new style returns new job.
+            _JOB_DEPRECATION_ISSUED = True
+            issue_deprecation_msg(
+                msg="Note that the 'job_id' and 'backend' attributes of "
+                "a runtime job have been deprecated",
+                version="0.7",
+                remedy="Please use the job_id() and backend() methods instead.",
+            )
+
         job = RuntimeJob(
             backend=backend,
             api_client=self._api_client,
@@ -927,10 +982,15 @@ class QiskitRuntimeService:
             params=inputs,
             user_callback=callback,
             result_decoder=result_decoder,
-            image=options.image,
+            image=qrt_options.image,
         )
         return job
 
+    @deprecate_function(
+        "QiskitRuntimeService.open",
+        "0.7",
+        "Instead, use the qiskit_ibm_runtime.Session class to create a runtime session.",
+    )
     def open(
         self,
         program_id: str,
@@ -1432,5 +1492,14 @@ class QiskitRuntimeService:
         """
         return self._channel
 
+    @property
+    def runtime(self):  # type:ignore
+        """Return self for compatibility with IBMQ provider.
+
+        Returns:
+            self
+        """
+        return self
+
     def __repr__(self) -> str:
         return "<{}>".format(self.__class__.__name__)
