@@ -16,6 +16,7 @@ import keyword
 import logging
 import os
 import re
+import hashlib
 from queue import Queue
 from threading import Condition
 from typing import List, Optional, Any, Dict, Union, Tuple, Type
@@ -108,7 +109,7 @@ def get_runtime_api_base_url(url: str, instance: str) -> str:
     api_host = url
 
     # cloud: compute runtime API URL based on crn and URL
-    if is_crn(instance):
+    if is_crn(instance) and not _is_experimental_runtime_url(url):
         parsed_url = urlparse(url)
         api_host = (
             f"{parsed_url.scheme}://{_location_from_crn(instance)}"
@@ -118,6 +119,16 @@ def get_runtime_api_base_url(url: str, instance: str) -> str:
     return api_host
 
 
+def _is_experimental_runtime_url(url: str) -> bool:
+    """Checks if the provided url points to an experimental runtime cluster.
+    This type of URLs is used for internal development purposes only.
+
+    Args:
+        url: The URL.
+    """
+    return isinstance(url, str) and "experimental" in url and url.endswith(".cloud")
+
+
 def _location_from_crn(crn: str) -> str:
     """Computes the location from a given CRN.
 
@@ -251,6 +262,13 @@ def _filter_value(
                 _filter_value(value, filter_keys)
 
 
+def _hash(hash_str: str) -> str:
+    """Hashes and returns a digest.
+    blake2s is supposedly faster than SHAs.
+    """
+    return hashlib.blake2s(hash_str.encode()).hexdigest()
+
+
 class RefreshQueue(Queue):
     """A queue that replaces the oldest item with the new item being added when full.
 
@@ -310,3 +328,10 @@ class RefreshQueue(Queue):
         """Wake up all threads waiting for items on the queued."""
         with self.condition:
             self.condition.notifyAll()
+
+
+class CallableStr(str):
+    """A callable string."""
+
+    def __call__(self) -> str:
+        return self
