@@ -36,7 +36,7 @@ class TestIntegrationResults(IBMIntegrationJobTestCase):
             if "iteration" in result:
                 final_it = result["iteration"]
             nonlocal callback_err
-            if job_id != job.job_id:
+            if job_id != job.job_id():
                 callback_err.append(f"Unexpected job ID: {job_id}")
             if "interim_results" in result and result["interim_results"] != int_res:
                 callback_err.append(f"Unexpected interim result: {result}")
@@ -67,7 +67,7 @@ class TestIntegrationResults(IBMIntegrationJobTestCase):
             if "iteration" in result:
                 final_it = result["iteration"]
             nonlocal callback_err
-            if job_id != job.job_id:
+            if job_id != job.job_id():
                 callback_err.append(f"Unexpected job ID: {job_id}")
             if "interim_results" in result and result["interim_results"] != int_res:
                 callback_err.append(f"Unexpected interim result: {result}")
@@ -98,7 +98,7 @@ class TestIntegrationResults(IBMIntegrationJobTestCase):
             if "iteration" in result:
                 final_it = result["iteration"]
             nonlocal callback_err
-            if job_id != job.job_id:
+            if job_id != job.job_id():
                 callback_err.append(f"Unexpected job ID: {job_id}")
             if "interim_results" in result and result["interim_results"] != int_res:
                 callback_err.append(f"Unexpected interim result: {result}")
@@ -169,11 +169,12 @@ class TestIntegrationResults(IBMIntegrationJobTestCase):
                 final_it = result["iteration"]
 
         final_it = 0
-        iterations = 3
+        iterations = 10
+        inputs = {"iterations": iterations, "sleep_per_iteration": 3}
         with self.assertLogs("qiskit_ibm_runtime", level="WARNING") as err_cm:
             job = self._run_program(
                 service,
-                iterations=iterations,
+                inputs=inputs,
                 interim_results="foo",
                 callback=result_callback,
             )
@@ -227,7 +228,7 @@ class TestIntegrationResults(IBMIntegrationJobTestCase):
         callback_called = False
 
         with use_proxies(service, MockProxyServer.VALID_PROXIES):
-            job = self._run_program(service, iterations=1, callback=result_callback)
+            job = self._run_program(service, iterations=10, callback=result_callback)
             job.wait_for_final_state()
 
         self.assertTrue(callback_called)
