@@ -15,7 +15,7 @@
 from datetime import datetime as python_datetime
 from typing import List, Dict, Any, Optional
 
-from qiskit.test.mock.backends import FakeLima
+from qiskit.providers.fake_provider import FakeLima
 
 
 class FakeApiBackend:
