@@ -13,12 +13,23 @@
 """Test for the Websocket client."""
 
 import time
+from unittest.mock import MagicMock
 
 from qiskit.providers.fake_provider import FakeQasmSimulator
-
-from qiskit_ibm_runtime import RuntimeJob
+from qiskit.test.reference_circuits import ReferenceCircuits
+from qiskit.quantum_info import SparsePauliOp
+from qiskit_ibm_runtime import (
+    RuntimeJob,
+    QiskitRuntimeService,
+    Sampler,
+    Estimator,
+    Session,
+    Options,
+)
 from qiskit_ibm_runtime.api.client_parameters import ClientParameters
 from qiskit_ibm_runtime.exceptions import RuntimeInvalidStateError
+from qiskit_ibm_runtime.ibm_backend import IBMBackend
+
 from .mock.fake_runtime_client import BaseFakeRuntimeClient
 from .mock.ws_handler import (
     websocket_handler,
@@ -65,6 +76,38 @@ class TestRuntimeWebsocketClient(IBMTestCase):
         self.assertEqual(JOB_PROGRESS_RESULT_COUNT, len(results))
         self.assertFalse(job._ws_client.connected)
 
+    def test_primitive_interim_result_callback(self):
+        """Test primitive interim result callback."""
+
+        def result_callback(job_id, interim_result):
+            nonlocal results
+            results.append(interim_result)
+            self.assertEqual(JOB_ID_PROGRESS_DONE, job_id)
+
+        def _patched_run(callback, *args, **kwargs):  # pylint: disable=unused-argument
+            return self._get_job(callback=callback, backend=MagicMock(spec=IBMBackend))
+
+        service = MagicMock(spec=QiskitRuntimeService)
+        service.run = _patched_run
+
+        circ = ReferenceCircuits.bell()
+        obs = SparsePauliOp.from_list([("IZ", 1)])
+        primitives = [Sampler, Estimator]
+        sub_tests = [
+            ({"options": Options(environment={"callback": result_callback})}, {}),
+            ({}, {"callback": result_callback}),
+        ]
+
+        for cls in primitives:
+            for options, callback in sub_tests:
+                with self.subTest(primitive=cls, options=options, callback=callback):
+                    results = []
+                    inst = cls(session=Session(service=service), **options)
+                    job = inst.run(circ, observables=obs, **callback)
+                    time.sleep(JOB_PROGRESS_RESULT_COUNT + 2)
+                    self.assertEqual(JOB_PROGRESS_RESULT_COUNT, len(results))
+                    self.assertFalse(job._ws_client.connected)
+
     def test_stream_results(self):
         """Test streaming results."""
 
@@ -184,13 +227,14 @@ class TestRuntimeWebsocketClient(IBMTestCase):
         self.assertEqual(0, len(results))
         self.assertFalse(job._ws_client.connected)
 
-    def _get_job(self, callback=None, job_id=JOB_ID_PROGRESS_DONE):
+    def _get_job(self, callback=None, job_id=JOB_ID_PROGRESS_DONE, backend=None):
         """Get a runtime job."""
         params = ClientParameters(
             channel="ibm_quantum", token="my_token", url=MockWsServer.VALID_WS_URL
         )
+        backend = backend or FakeQasmSimulator()
         job = RuntimeJob(
-            backend=FakeQasmSimulator(),
+            backend=backend,
             api_client=BaseFakeRuntimeClient(),
             client_params=params,
             job_id=job_id,
